/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: SemanticPersistence.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.semantic.persistence;

import java.util.Set;
import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import org.openrdf.repository.Repository;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SemanticPersistence.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
public interface SemanticPersistence
  {
    public final class Locator
      {
        private Locator()
          {
          }

        /***********************************************************************
         *
         * Gets a reference to the CatalogNodeManager.
         *
         **********************************************************************/
        @Nonnull
        public static final SemanticPersistence findSemanticPersistence()
          {
            final Lookup lookup = Lookup.getDefault();
            final SemanticPersistence semanticPersistence = lookup.lookup(SemanticPersistence.class);

            if (semanticPersistence == null)
              {
                throw new RuntimeException("Cannot lookup SemanticPersistence");
              }

            return semanticPersistence;
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Repository createRepository (@Nonnull final String name);

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void removeRepository (@Nonnull final Repository context);

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Set<String> getRepositoryNames();

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Repository findRepositoryByName (@Nonnull final String name)
      throws NoSuchContextException;
  }
