/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: DataSourceStore.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.semantic.persistence.impl;

import it.tidalwave.openide.persistence.TxTask;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.util.Iterator;
import javax.imageio.spi.ServiceRegistry;
import javax.sql.DataSource;
import org.openrdf.sail.SailException;
import org.openrdf.sail.rdbms.RdbmsConnectionFactory;
import org.openrdf.sail.rdbms.RdbmsProvider;
import org.openrdf.sail.rdbms.RdbmsStore;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: DataSourceStore.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
public class DataSourceStore extends RdbmsStore
  {
    private final DataSource dataSource;

    public DataSourceStore (final DataSource dataSource)
      {
        this.dataSource = dataSource;
      }

    @Override
    public void initialize()
      throws SailException
      {
        TxTask.execute(SailException.class, new TxTask<Void, SailException>()
          {
            public Void run()
              throws SailException
              {
                try
                  {
                    RdbmsConnectionFactory factory;
                    factory = createFactory();
                    factory.setMaxNumberOfTripleTables(getMaxNumberOfTripleTables());
                    factory.setTriplesIndexed(isIndexed());
                    factory.setSequenced(isSequenced());
                    factory.init();
                  }
                catch (Exception e)
                  {
                    throw new SailException(e);
                  }
                
                return null;
              }
          });
      }

	private RdbmsConnectionFactory createFactory()
      throws Exception
  	  {
        Connection con = dataSource.getConnection();

        try
          {
            DatabaseMetaData metaData = con.getMetaData();
            RdbmsConnectionFactory factory = newFactory(metaData);
            factory.setSail(DataSourceStore.this);
            factory.setDataSource(dataSource);

            return factory;
          }
        finally
          {
            con.close();
          }
	  }

	private RdbmsConnectionFactory newFactory (DatabaseMetaData metaData)
      throws SQLException
      {
		String dbn = metaData.getDatabaseProductName();
		String dbv = metaData.getDatabaseProductVersion();
		RdbmsConnectionFactory factory;
		Iterator<RdbmsProvider> providers;
		providers = ServiceRegistry.lookupProviders(RdbmsProvider.class);
	
        while (providers.hasNext())
          {
			RdbmsProvider provider = providers.next();
			factory = provider.createRdbmsConnectionFactory(dbn, dbv);

            if (factory != null)
				return factory;
		  }

		return new RdbmsConnectionFactory();
	  }
  }
