/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: SemanticPersistenceImpl.java,v 2bd58fe8e505 2010/01/25 15:55:13 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.semantic.persistence.impl;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.io.File;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.sail.memory.MemoryStore;
import org.openide.util.Parameters;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.openide.persistence.Persistence;
import it.tidalwave.netbeans.workspacemanager.WorkspaceManager;
import it.tidalwave.semantic.persistence.NoSuchContextException;
import it.tidalwave.semantic.persistence.SemanticPersistence;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SemanticPersistenceImpl.java,v 2bd58fe8e505 2010/01/25 15:55:13 fabrizio $
 *
 ******************************************************************************/
public class SemanticPersistenceImpl implements SemanticPersistence
  {
    private final static String CLASS = SemanticPersistenceImpl.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);
    
    private final Map<String, Repository> repositoryMapByName = new HashMap<String, Repository>();
    private final List<Repository> repositories = new ArrayList<Repository>();

    public SemanticPersistenceImpl()
      {
        try
          {
            final WorkspaceManager workspaceManager = it.tidalwave.netbeans.util.Locator.find(WorkspaceManager.class);
            final File openRDFFolder = new File(workspaceManager.getWorkspaceFolder(), "OpenRDF");
            final File folder = new File(openRDFFolder, "DEFAULT");
            folder.mkdirs();
            Repository repository;
            
            try
              {
                final Persistence persistence = Persistence.Locator.findPersistence();
                final ProxyDataSource dataSource = new ProxyDataSource(persistence.getDataSource());
                repository = new SailRepository(new DataSourceStore(dataSource));
                repository.initialize();
                repositories.add(repository);
              }
            catch (Exception e) // FIXME: temporarily until Derby works
              {
                logger.throwing(CLASS, "ctor", e);
                logger.severe("Cannot run Derby repository, using memory store");
                repository = new SailRepository(new MemoryStore(folder));
                repository.initialize();
                repositories.add(repository);
              }

            repositoryMapByName.put("DEFAULT", repository);
          }
        catch (Exception e)
          {
            throw new ExceptionInInitializerError(e);
          }
      }

    public Repository createRepository(String name)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public void removeRepository(Repository repository)
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    @Nonnull
    public Set<String> getRepositoryNames()
      {
        return new TreeSet<String>(repositoryMapByName.keySet());
      }

    @Nonnull
    public Repository findRepositoryByName (@Nonnull final String name)
      throws NoSuchContextException
      {
        Parameters.notNull("name", name);

        final Repository context = repositoryMapByName.get(name);

        if (context == null)
          {
            throw new NoSuchContextException(name);
          }

        return context;
      }

    public void shutdown()
      {
        for (final Repository repository : repositories)
          {
            try
              {
                repository.shutDown();
              }
            catch (RepositoryException e)
              {
                e.printStackTrace(); // FIXME
              }
          }
      }
  }
